#!/usr/bin/env jython
"""
Misc.py - 0.7 - 2007-10-21

IBM MQ related functions for Jython >= 2.2.1.

Copyright (C) 2007  Gert-Jan den Besten

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>   
"""

from __future__ import generators    # Remove this statement when using Jython 2.3

# Java/MQ imports
from com.ibm.mq import MQEnvironment, MQException, MQC, MQMessage, MQQueueManager, \
                       MQQueue, MQPutMessageOptions, MQGetMessageOptions

# Jython imports
import jarray

# Python imports
from Message import JyMQMessage




def BrowseFirstMessage(objQueue):
    """
    Browse first message on queue.
    """
    objMessage     = MQMessage()
    objBMO         = MQGetMessageOptions()
    objBMO.options = objBMO.options | MQC.MQGMO_BROWSE_FIRST

    objQueue.get(objMessage, objBMO)

    return JyMQMessage(objMessage)



def BrowseNextMessage(objQueue):
    """
    Browse next message on queue.
    """
    objMessage     = MQMessage()
    objBMO         = MQGetMessageOptions()
    objBMO.options = objBMO.options | MQC.MQGMO_BROWSE_NEXT

    objQueue.get(objMessage, objBMO)

    return JyMQMessage(objMessage)



def BrowseAllMessages(objQueue):
    """
    This Generator browses all messages on objQueue.
    Raises StopIteration.
    """
    try:
        yield BrowseFirstMessage(objQueue)
        while True:
            yield BrowseNextMessage(objQueue)
    except MQException:
        raise StopIteration



def GetMessage(objQueue):
    """
    Get message from Queue.
    """
    objMessage = MQMessage()
    objGMO     = MQGetMessageOptions()

    objQueue.get(objMessage, objGMO)

    return JyMQMessage(objMessage)



def GetAllMessages(objQueue):
    """
    This Generator gets all messages from objQueue.
    Raises StopIteration.
    """
    try:
        while True:
            yield GetMessage(objQueue)
    except MQException:
        raise StopIteration



def PutMessage(objMessage, objQueue):
    """
    Put message on queue.
    """
    objPMO = MQPutMessageOptions()
    objQueue.put(objMessage, objPMO)




def OpenQmgr(strQmgr, dctSettings):
    """
    Trying to connect to the queuemanager.
    """
    MQEnvironment.hostname = dctSettings['host']
    MQEnvironment.channel  = dctSettings['channel']
    MQEnvironment.port     = dctSettings['port']

    try:
        objQueueManager = MQQueueManager(strQmgr)
    except MQException:
        raise EnvironmentError('Failed to connect to queuemanager.')

    return objQueueManager



def OpenQueue(objQueueManager, strQueueName):
    """
    Trying to open queue strQueueName within objQueueManager.
    """
    intOpenOptions = (MQC.MQOO_INPUT_AS_Q_DEF |
                      MQC.MQOO_PASS_ALL_CONTEXT |
                      MQC.MQOO_OUTPUT |
                      MQC.MQOO_INQUIRE |
                      MQC.MQOO_BROWSE)
    try:
        objQueue = objQueueManager.accessQueue(strQueueName, intOpenOptions)
    except MQException:
        raise EnvironmentError('Failed to open queue.')

    return objQueue



def ClearQueue(objQueue):
    """
    Clearing objQueue by reading all its messages.
    """
    for myGetMessage in GetAllMessages(objQueue):
        pass



def Mq24ByteField(strIdent):
    """
    Put first 24 characters of strIdent into a java byte-array.
    """
    return jarray.array([ord(char) for char in strIdent[:24]], 'b')



def _test():
    pass




if __name__ == "__main__":
    pass
